<?php

namespace Litecms\App\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Litecms\App\Forms\Apps as AppsForm;
use Litecms\App\Http\Requests\AppsRequest;
use Litecms\App\Http\Resources\AppsResource;
use Litecms\App\Http\Resources\AppsCollection;
use Litecms\App\Models\Apps;
use Litecms\App\Scopes\AppsResourceScope;


/**
 * Resource controller class for apps.
 */
class AppsResourceController extends BaseController
{

    /**
     * Initialize apps resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = AppsForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('litecms.app.modules'), 'app', guard_url('app'));
    }

    /**
     * Display a list of apps.
     *
     * @return Response
     */
    public function index(AppsRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Apps::pushScope(new RequestScope())
            ->pushScope(new AppsResourceScope())
            ->simplePaginate($pageLimit);

        $data = new AppsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('app::apps.names'))
            ->view('app::apps.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display apps.
     *
     * @param Request $request
     * @param Model   $apps
     *
     * @return Response
     */
    public function show(AppsRequest $request, Apps $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AppsResource($model);

        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('app::apps.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('app::apps.show')
            ->output();

    }

    /**
     * Show the form for creating a new apps.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AppsRequest $request, Apps $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AppsResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('app::apps.name'))
            ->view('app::apps.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new apps.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AppsRequest $request, Apps $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new AppsResource($model);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('app::apps.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('app/apps/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/app/apps'))
                ->redirect();
        }

    }

    /**
     * Show apps for editing.
     *
     * @param Request $request
     * @param Model   $apps
     *
     * @return Response
     */
    public function edit(AppsRequest $request, Apps $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new AppsResource($model);

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('app::apps.name'))
            ->view('app::apps.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the apps.
     *
     * @param Request $request
     * @param Model   $apps
     *
     * @return Response
     */
    public function update(AppsRequest $request, Apps $model)
    {
        try {
            $attributes = $request->all();
            $model = $model->update($attributes);
            $data = new AppsResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('app::apps.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/apps/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/apps/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the apps.
     *
     * @param Model   $apps
     *
     * @return Response
     */
    public function destroy(AppsRequest $request, Apps $model)
    {
        try {
            $model->delete();
            $data = new AppsResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('app::apps.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/apps/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/apps/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
