<?php

namespace Litecms\App\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Litecms\App\Forms\Permission as PermissionForm;
use Litecms\App\Http\Requests\PermissionRequest;
use Litecms\App\Http\Resources\PermissionResource;
use Litecms\App\Http\Resources\PermissionsCollection;
use Litecms\App\Models\Permission;
use Litecms\App\Scopes\PermissionResourceScope;


/**
 * Resource controller class for permission.
 */
class PermissionResourceController extends BaseController
{

    /**
     * Initialize permission resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = PermissionForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('litecms.app.modules'), 'app', guard_url('app'));
    }

    /**
     * Display a list of permission.
     *
     * @return Response
     */
    public function index(PermissionRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Permission::pushScope(new RequestScope())
            ->pushScope(new PermissionResourceScope())
            ->simplePaginate($pageLimit);

        $data = new PermissionsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('app::permission.names'))
            ->view('app::permission.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display permission.
     *
     * @param Request $request
     * @param Model   $permission
     *
     * @return Response
     */
    public function show(PermissionRequest $request, Permission $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PermissionResource($model);

        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('app::permission.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('app::permission.show')
            ->output();

    }

    /**
     * Show the form for creating a new permission.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PermissionRequest $request, Permission $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PermissionResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('app::permission.name'))
            ->view('app::permission.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new permission.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PermissionRequest $request, Permission $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new PermissionResource($model);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('app::permission.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('app/permission/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/app/permission'))
                ->redirect();
        }

    }

    /**
     * Show permission for editing.
     *
     * @param Request $request
     * @param Model   $permission
     *
     * @return Response
     */
    public function edit(PermissionRequest $request, Permission $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PermissionResource($model);

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('app::permission.name'))
            ->view('app::permission.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the permission.
     *
     * @param Request $request
     * @param Model   $permission
     *
     * @return Response
     */
    public function update(PermissionRequest $request, Permission $model)
    {
        try {
            $attributes = $request->all();
            $model = $model->update($attributes);
            $data = new PermissionResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('app::permission.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/permission/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/permission/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the permission.
     *
     * @param Model   $permission
     *
     * @return Response
     */
    public function destroy(PermissionRequest $request, Permission $model)
    {
        try {
            $model->delete();
            $data = new PermissionResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('app::permission.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/permission/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/permission/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
