<?php

namespace Litecms\App\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Litecms\App\Forms\User as UserForm;
use Litecms\App\Http\Requests\UserRequest;
use Litecms\App\Http\Resources\UserResource;
use Litecms\App\Http\Resources\UsersCollection;
use Litecms\App\Models\User;
use Litecms\App\Scopes\UserResourceScope;


/**
 * Resource controller class for user.
 */
class UserResourceController extends BaseController
{

    /**
     * Initialize user resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = UserForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('litecms.app.modules'), 'app', guard_url('app'));
    }

    /**
     * Display a list of user.
     *
     * @return Response
     */
    public function index(UserRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = User::pushScope(new RequestScope())
            ->pushScope(new UserResourceScope())
            ->simplePaginate($pageLimit);

        $data = new UsersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('app::user.names'))
            ->view('app::user.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display user.
     *
     * @param Request $request
     * @param Model   $user
     *
     * @return Response
     */
    public function show(UserRequest $request, User $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new UserResource($model);

        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('app::user.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('app::user.show')
            ->output();

    }

    /**
     * Show the form for creating a new user.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(UserRequest $request, User $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new UserResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('app::user.name'))
            ->view('app::user.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new user.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(UserRequest $request, User $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new UserResource($model);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('app::user.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('app/user/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/app/user'))
                ->redirect();
        }

    }

    /**
     * Show user for editing.
     *
     * @param Request $request
     * @param Model   $user
     *
     * @return Response
     */
    public function edit(UserRequest $request, User $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new UserResource($model);

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('app::user.name'))
            ->view('app::user.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the user.
     *
     * @param Request $request
     * @param Model   $user
     *
     * @return Response
     */
    public function update(UserRequest $request, User $model)
    {
        try {
            $attributes = $request->all();
            $model = $model->update($attributes);
            $data = new UserResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('app::user.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/user/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/user/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the user.
     *
     * @param Model   $user
     *
     * @return Response
     */
    public function destroy(UserRequest $request, User $model)
    {
        try {
            $model->delete();
            $data = new UserResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('app::user.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('app/user/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('app/user/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
